;**********************************************************************
;    Filename:	    Exp12.asm                                        *
;    Date:          07/04/2002                                        *
;    Author: KRITSADA JAIYEN                                          *
;    Company: Innovative Experiment                                   *
;**********************************************************************
;    Files required: p16f628.inc                                      *
;**********************************************************************
;    Notes: Minute CountDown And Drive Relay	                      *
;**********************************************************************

	list      p=16f628      ; list directive to define processor
	#include <p16f628.inc>  ; processor specific variable definitions
	
	__CONFIG _CP_OFF & _WDT_OFF & _BODEN_ON & _PWRTE_ON & _XT_OSC & _MCLRE_ON & _LVP_OFF

	CBLOCK	0x20
DUMMY2	
COUNT	
DIGIT1	
DIGIT2	
SECOND	
FLAG_	
SEND	
MIN	
W_TEMP 
S_TEMP 
	ENDC

#define	DAT_PIN	PORTB,0		; 74595 DATA pin
#define LATCH	PORTB,4		; 74595 LATCH pin 
#define CLK	PORTB,5		; 74595 CLK pin
#Define	DSP1	PORTB,6		; DSP1 common
#Define	DSP2	PORTB,7		; DSP2 common
#Define KEY_IN	PORTA,3		; Start switch

	org	0x000		
	goto	Init	

	org	0x004		; Interrupt vector
	goto	Isr

Table_	addwf	PCL,f		; Table for LED 7 segment
	retlw	0x3f		; 0
	retlw	0x06		; 1
	retlw	0x5b		; 2
	retlw	0x4f		; 3
	retlw	0x66		; 4
	retlw	0x6d		; 5	
	retlw	0x7d		; 6
	retlw	0x07		; 7
	retlw	0x7f		; 8
	retlw	0x6f		; 9
	return		

Init	clrf 	PORTA		; Initial PORTA to digital I/O by
	movlw	0x07		; disable analog comparator
	movwf 	CMCON		; 
	bsf	STATUS,RP0
	movlw	b'00001110'	; RB0, RB4-RB7 as output 
	movwf	PORTB
	movlw	b'11101111'	; RA4 as output for driving relay and RA3 as input for detecting switch
	movwf	PORTA
	movlw	b'11000010'	; Defind prescaler rate to internal clock 1:8 
	movwf	OPTION_REG&7f
	bcf	STATUS,RP0

	bsf	PORTA,4		; Relay off
	clrf	FLAG_		; Clear flag for checking 0.5 second timing		
	
Main	clrf	DIGIT2		; Load timer value with 2 minutes
	movlw	0x02
	movwf	DIGIT1
Key	call	Disp		; Show time on LED display
	btfsc	KEY_IN		; Checking START switch at RA3
	goto	Key

	bcf	INTCON,T0IF	; Set Timer0 interrupt
	bsf	INTCON,T0IE
	bsf	INTCON,GIE

	movlw	0x07		; Load value for setting 500 millisecond time period
	movwf	TMR0
	movlw	0x06	
	movwf	SECOND
	movlw	.59		; Load value for setting 60 second time period
	movwf	MIN
	bcf	PORTA,4		; Turn relay on and begin counting
Loop	call	Disp		; Main loop for timing display
	btfss	PORTA,4		; Test relay off ?
	goto	Loop		; If no, do loop again
	goto	Main		; If yes, start again

;**********************************************************************
;                     SubRoutine 				      *	
;**********************************************************************
Disp	movf	DIGIT1,w		; Load DSP1
	call	Table_		; Get LED data from table
	btfsc	FLAG_,7		; Blinking dot-point
	addlw	0x80	
	call	Send_Dat		; Send data to 74HC595
	bcf	DSP1		; On DSP1
	call	Delay2		; 
	bsf	DSP1		; Off DSP1

	movf	DIGIT2,w		; Load DSP2
	call	Table_		; Get LED data from table
	call	Send_Dat		; Send data to 74HC595
	bcf	DSP2		; On DSP2
	call	Delay2		; 
	bsf	DSP2		; Off DSP2
	return	
	
Send_Dat movwf	SEND		; Load data
	movlw	0x08		; Loop 8 times for sending 8-bit data
	movwf	COUNT
Chk_Bit	bcf	DAT_PIN		; RB0 as DATA pin of 74HC595
	btfsc	SEND,7		; Check bit 7 "0" or "1"	
	bsf	DAT_PIN		; Send bit 7 to shift register
Clock	bsf	CLK		; Clock pulse
	bcf	CLK		
Rotate	rlf	SEND,f		; Rotate bit for sending next bit
	decfsz	COUNT,f		; Test loop 8 times ?
	goto	Chk_Bit		; If no, send bit again
Latch	bsf	LATCH		; Latch data for displaying 
	bcf	LATCH
	return


Delay2	clrf	DUMMY2		; Multiplexing delay time
	decfsz	DUMMY2,f
	goto	$-1	
	return

;**********************************************************************
;       Interrupt Service Routine  				      *	
;**********************************************************************
Isr	movwf   W_TEMP          	; Save off current W register contents
	movf	STATUS,w        	; Move status register into W register
	movwf	S_TEMP       	; Save off contents of STATUS register

	bcf	INTCON,T0IF	; Clear Timer0 interrupt flag
	movlw	0x07		; Load TMR0 again	
	movwf	TMR0
	incfsz	SECOND,f	; Increment & test SECOND register to 0 ?	
	goto	Ret2		; If SECOND <> 0 then return
	movlw	0x06		; If SECOND = 0, load data to SECOND register again
	movwf	SECOND
	movlw	0x80		; Toggle FLAG_ register
	xorwf	FLAG_,f
	btfsc	FLAG_,7		; Test FLAG_ = "0" or "1"
	goto	Ret2		; If FLAG_ = "1" then return
				; If FLAG_ = "0" , Timer = 1 then decrease minute value
	decfsz	MIN,f		; 1 minute checking
	goto	Ret2		; If <> 1 minute then return
	movlw	.59		; Load value for setting 60 seconds again
	movwf	MIN

;------Check & Decrese timer ------------------------
St_test	movf	DIGIT1,f		; Check DSP1 = 0 ?
	btfsc	STATUS,Z
	goto	Dec_1		; If yes, check another
	decfsz	DIGIT1,f		; If no, decrease & check decrement = 0 ?
	goto	Ret2		; If no, return
	

Dec_1	movf	DIGIT2,f		; If yes, check DSP2
	btfsc	STATUS,Z
	goto	Zero_		; if DSP1 & DSP2 = 0, clear system 
	decf	DIGIT2,f		; If DSP2 <> 0, decrease timer
	movlw	0x09		; Load "09" to DSP1
	movwf	DIGIT1
	goto	Ret2		; return
	
Zero_	bcf	INTCON,T0IE	; Disable interrupt
	bsf	PORTA,4		; Off relay

Ret2	movf    S_TEMP,w     	; Retrieve copy of STATUS register
	movwf	STATUS         	; Restore pre-isr STATUS register contents
	swapf   W_TEMP,f
	swapf   W_TEMP,w        	; Restore pre-isr W register contents
	retfie                 		; Return from interrupt

	END                    	

