;**********************************************************************
;                                                                     *
;    Filename:	    Exp13.asm                                        *
;    Date:          08/04/2002                                        *
;    Author: KRITSADA JAIYEN                                          *
;    Company: Innovative Experiment                                   *
;**********************************************************************
;    Files required: p16f628.inc                                      *
;**********************************************************************
;    Notes: Timer UpDate 			                      *
;**********************************************************************

	list      p=16f628    	; list directive to define processor
	#include <p16f628.inc> 	; processor specific variable definitions
	errorlevel 0,-302	; Disbles Bank Warnning Message	
	__CONFIG _CP_OFF & _WDT_OFF & _BODEN_ON & _PWRTE_ON & _XT_OSC & _MCLRE_ON & _LVP_OFF

	CBLOCK	0x20
DUMMY3	
COUNT	
DIGIT1	
DIGIT2	
DG1_SAVE
DG2_SAVE
SECOND	
FLAG_	
SEND	
DUMMY1	
MIN	
	ENDC

#define	DAT_PIN	PORTB,0		; 74595 DATA pin
#define LATCH	PORTB,4		; 74595 LATCH pin 
#define CLK	PORTB,5		; 74595 CLK pin
#Define	DSP1	PORTB,6		; Digit1 common
#Define	DSP2	PORTB,7		; Digit2 common
#Define KEY_IN	PORTA,3		; Start switch
#Define INC1	PORTA,0		; Increment switch for 10's unit
#Define INC2	PORTA,1		; Increment switch for 1's unit

W_TEMP  equ     0x70        	; Variable used for context saving 
S_TEMP  equ     0x71        	; Variable used for context saving
	
	org	0x000
	goto	Init

	org	0x004
	goto	Isr

Table_	addwf	PCL,f
	dt	0x3f,0x06,0x5b,0x4f,0x66,0x6d,0x7d,0x07,0x7f,0x6f

Init	clrf 	PORTA		; Initial PORTA  		
	movlw	0x07		; Turn comparator off and
	movwf 	CMCON		; enable all pins for I/O
	bsf	STATUS,RP0
	movlw	b'00001110'	; RB0,RB4-RB7 as output 
	movwf	PORTB
	bcf	PORTA,4		; RA4 as output, RA3 as input
	movlw	b'11000010'
	movwf	OPTION_REG
	bcf	STATUS,RP0

	bsf	PORTA,4		; Off relay
	clrf	FLAG_		; Clear dot-point flag

	clrf	DIGIT1		; Start with 00 on LED display
	clrf	DIGIT2

Test_Key call	Delay		; Key bounce delay & display	
	btfss	INC1		; DSP1 increment if press
	goto	Inc_Dg1		; 
	btfss	INC2		; DSP2 increment if press
	goto	Inc_Dg2		; 
	btfss	KEY_IN		; Running if press start switch
	goto	Start		; 
	goto	Test_Key		; Check switch pressing again	

Inc_Dg1	movlw	0x09		; For S1 pressing, test DSP1 = "9" ?
	subwf	DIGIT1,w		; 
	btfss	STATUS,Z
	goto	Inc_1		; If no, normal increment
	clrf	DIGIT1		; Clear DSP1 if DSP1 = "9" ?
	goto	Test_Key		; Back to check switch again	
Inc_1	incf	DIGIT1,f
	goto	Test_Key

Inc_Dg2	movlw	0x09		; For S2 pressing, test DSP2 = "9" ?
	subwf	DIGIT2,w	
	btfss	STATUS,Z
	goto	Inc_2		; If no, Increment Normal
	clrf	DIGIT2		; Clear DSP2 if DSP2 = "9"
	goto	Test_Key		; Back to check switch again
Inc_2	incf	DIGIT2,f
	goto	Test_Key

Start	movf	DIGIT1,w		; Save data to file register for running next time
	movwf	DG1_SAVE
	movf	DIGIT2,w
	movwf	DG2_SAVE
	bcf	INTCON,T0IF	; Enable Timer0 interrupt
	bsf	INTCON,T0IE
	bsf	INTCON,GIE

Main	movlw	0x07		; Load Timer0 value
	movwf	TMR0
	movlw	0x06	
	movwf	SECOND
	movlw	.59
	movwf	MIN
	bcf	PORTA,4		; On relay 
Loop	call	Disp		; Show display	
	btfsc	INTCON,T0IE	; Check interrupt
	goto	Loop	
	movf	DG1_SAVE,w	; Read saving data from file register
	movwf	DIGIT1
	movf	DG2_SAVE,w
	movwf	DIGIT2
	goto	Test_Key	; Back To Test Key Again

;**********************************************************************
;  	Interupt Service Routine				      *	
;**********************************************************************

Isr	movwf   W_TEMP         	 ; Save current W register contents
	movf	STATUS,w       	 ; Move status register into W register
	movwf	S_TEMP       	; Save contents of STATUS register
	bcf	INTCON,T0IF	; Clear interrupt flag
	movlw	0x07		; Load new value again
	movwf	TMR0
	incfsz	SECOND,f	; Test 500 millisecond
	goto	Ret2		; If not, return
	movlw	0x06		; If yes, load new data again
	movwf	SECOND
	movlw	0x80		; Toggle flag
	xorwf	FLAG_,f
	btfsc	FLAG_,7		; Test dot-point "on" or "off"
	goto	Ret2

	decfsz	MIN,f		; 1 minute checking
	goto	Ret2
	movlw	.59		; Load value to set 60 second again
	movwf	MIN

;------Check & decrement timer ------------------------

St_test	movf	DIGIT1,f		; Check DSP1 = 0 ?
	btfsc	STATUS,Z
	goto	Dec_1		; If yes, check DSP2
	decfsz	DIGIT1,f		; If no, decrement 	
	goto	Ret2
	

Dec_1	movf	DIGIT2,f		; Check DSP2 = 0 ?
	btfsc	STATUS,Z
	goto	Zero_		; DSP1 and DSP2 = 0
	decf	DIGIT2,f		; If DSP2 <> 0, decrement and load "9" for DSP1
	movlw	0x09		
	movwf	DIGIT1
	goto	Ret2
	
Zero_	bcf	INTCON,T0IE	; Disable Timer0 interrupt
	bsf	PORTA,4		; Off relay
Ret2	movf    S_TEMP,w     	; Retrieve copy of STATUS register
	movwf	STATUS         	; Restore pre-isr STATUS register contents
	swapf   W_TEMP,f
	swapf   W_TEMP,w        ; Restore pre-isr W register contents
	retfie                  ; Return from interrupt

;**********************************************************************
;  	SubRoutine						      *	
;**********************************************************************
Disp	movf	DIGIT1,w		; Load DSP1 data display 
	call	Table_		; Get LED data from table
	btfsc	FLAG_,7		; Blink dot-point of DSP1
	addlw	0x80	
	call	Send_Dat		; Send data to 74HC595
	bcf	DSP1		; On DSP1
	call	Delay2		; Dispaly time
	bsf	DSP1		; Off DSP1

	movf	DIGIT2,w		; Load DSP2 data display 
	call	Table_		; Get LED data from table
	call	Send_Dat		; Send data to 74HC595
	bcf	DSP2		; On DSP2
	call	Delay2		; Display time
	bsf	DSP2		; Off DSP2
	return	

Send_Dat movwf	SEND		; Load data
	movlw	0x08		; Loop 8 times for sending 8-bit data
	movwf	COUNT
Chk_Bit	bcf	DAT_PIN		; RB0 connect DATA pin of 74HC595
	btfsc	SEND,7		; Check bit 7 ? 
	bsf	DAT_PIN		; Send bit 7 to shift register
Clock	bsf	CLK		; Clock pulse
	bcf	CLK		
Rotate	rlf	SEND,f		; Rotate bit for sending next bit
	decfsz	COUNT,f		; Test loop 8 times ?
	goto	Chk_Bit		; If no, send bit again
Latch	bsf	LATCH		; Latch data for showing
	bcf	LATCH
	return

Delay	movlw	0x50		; This delay use for waiting switch press		
	movwf	DUMMY1		; and debounce
	call	Disp
	decfsz	DUMMY1,f
	goto	$-2
	return	

Delay2	clrf	DUMMY3		; Multiplexing time
	decfsz	DUMMY3,f
	goto	$-1	
	return
					

