;**********************************************************************
;    Filename:	    Exp14.asm                                        *
;    Date:          09/04/2002                                        *
;    Author: KRITSADA JAIYEN                                          *
;    Company: Innovative Experiment                                   *
;**********************************************************************
;    Files required: p16f628.inc                                      *
;**********************************************************************
;    Notes: 99 minute exclusive timer			                      *
;**********************************************************************
	list      p=16f628     	; list directive to define processor
	#include <p16f628.inc>  ; processor specific variable definitions
	errorlevel 0,-302
	__CONFIG _CP_OFF & _WDT_OFF & _BODEN_ON & _PWRTE_ON & _XT_OSC & _MCLRE_ON & _LVP_OFF

	CBLOCK	0x20		
COUNT	
DIGIT1	
DIGIT2	
DG1_SAVE
DG2_SAVE
SECOND	
FLAG_	
SEND	
DUMMY1	
DUMMY3
MIN	
W_TEMP  
S_TEMP  
	ENDC

#define	DAT_PIN	PORTB,0		; 74595 DATA pin
#define LATCH	PORTB,4		; 74595 LATCH pin 
#define CLK	PORTB,5		; 74595 CLK pin
#Define	DSP1	PORTB,6		; DSP1 common
#Define	DSP2	PORTB,7		; DSP2 common
#Define KEY_IN	PORTA,3		; Start switch
#Define INC1	PORTA,0		; Increment switch for 10's unit
#Define INC2	PORTA,1		; Increment switch for 1's unit

	org	0x000	
	goto	Init

	org	0x004		; Interrupt vector
	goto	Isr

Table_	addwf	PCL,f
	dt	0x3f,0x06,0x5b,0x4f,0x66,0x6d,0x7d,0x07,0x7f,0x6f

Init	clrf 	PORTA		; Initial PORTA by setting output data latch 		
	movlw	0x07		; Turn comparator off and select to digital I/O
	movwf 	CMCON		; 
	bsf	STATUS,RP0
	movlw	b'00001110'	; RB0,RB4-RB7 as output
	movwf	PORTB
	bcf	PORTA,4		; RA4 as output, RA3,RA1,RA0 as input
	movlw	b'11000010'
	movwf	OPTION_REG
	bcf	STATUS,RP0

	bsf	PORTA,4		; Off relay
	clrf	FLAG_		; Clear dot-point flag	

	movlw	0x00
	call	EE_Read		; Get DSP1 data from EEPROM
	movwf	DIGIT1
	movlw	0x01		; Get DSP2 data from EEPROM
	call	EE_Read
	movwf	DIGIT2

Test_Key call	Delay		; Key bounce and display delay 	
	btfss	INC1		; DSP1 increment if press RA0 switch
	goto	Inc_Dg1		; 
	btfss	INC2		; DSP2 increment if press RA1 switch
	goto	Inc_Dg2		; 
	btfss	KEY_IN		; Running if press RA3 switch
	goto	Start		; 
	goto	Test_Key		; If not press, check switch again	

Inc_Dg1	movlw	0x09		; For S1 pressed, 
	subwf	DIGIT1,w		; test DSP1 = "9" ?
	btfss	STATUS,Z
	goto	Inc_1		; If no, normal increment
	clrf	DIGIT1		; If yes, clear DSP1
	goto	Test_Key		; Back to check switch again	
Inc_1	incf	DIGIT1,f
	goto	Test_Key

Inc_Dg2	movlw	0x09		; For S2 pressed,
	subwf	DIGIT2,w		; test DSP2 = "9" ?
	btfss	STATUS,Z
	goto	Inc_2		; If no, normal increment
	clrf	DIGIT2		; If yes, clear DSP2
	goto	Test_Key		; Back to check switch again
Inc_2	incf	DIGIT2,f
	goto	Test_Key

Start	movf	DIGIT1,w		; Save DSP1 to EEPROM
	bsf	STATUS,RP0
	movwf	EEDATA
	movlw	0x00
	movwf	EEADR
	call	EE_Write
	movf	DIGIT2,w		; Save DSP2 to EEPROM
	bsf	STATUS,RP0
	movwf	EEDATA
	movlw	0x01
	movwf	EEADR
	call	EE_Write
	bcf	INTCON,T0IF	; Enable Timer0 interrupt
	bsf	INTCON,T0IE
	bsf	INTCON,GIE

Main	movlw	0x07		; Load timer value
	movwf	TMR0
	movlw	0x06	
	movwf	SECOND
	movlw	.59
	movwf	MIN
	bcf	PORTA,4		; On relay

Loop	call	Disp		; Display on LED 7 segment	
	btfsc	INTCON,T0IE	; Check Timer0 interrupt
	goto	Loop	
	movlw	0x00		; Get data from EEPROM
	call	EE_Read		; 
	movwf	DIGIT1
	movlw	0x01
	call	EE_Read
	movwf	DIGIT2
	goto	Test_Key		; Back to check switch again


;**********************************************************************
;  	Interupt Service Routine				      *	
;**********************************************************************

Isr	movwf   W_TEMP         	; Save current W register contents
	movf	STATUS,w        	; Move status register into W register
	movwf	S_TEMP       	; Save contents of STATUS register
	bcf	INTCON,T0IF	; Clear interrupt flag
	movlw	0x07		; Load new value again
	movwf	TMR0
	incfsz	SECOND,f	; Test 500 millisecond ?
	goto	Ret2		; If no,  return
	movlw	0x06		; If yes, load new value again
	movwf	SECOND
	movlw	0x80		; Toggle flag_
	xorwf	FLAG_,f
	btfsc	FLAG_,7		; Test dot-point "ON" or "OFF"
	goto	Ret2

	decfsz	MIN,f		; 1 minute checking
	goto	Ret2
	movlw	.59		; Load value for setting 60 second again
	movwf	MIN

;------Check & Decrement Timer ------------------------

St_test	movf	DIGIT1,f	; Check Digit1 = 0 ?
	btfsc	STATUS,Z
	goto	Dec_1		; If DSP1 = "0", check DSP2
	decfsz	DIGIT1,f		; If no, decrement DSP1 	
	goto	Ret2
	

Dec_1	movf	DIGIT2,f		
	btfsc	STATUS,Z	; Check DSP2 = "0" ?
	goto	Zero_		; If yes, goto finishing routine
	decf	DIGIT2,f		; If no, decrease DSP2
	movlw	0x09		; and load "9" to DSP1
	movwf	DIGIT1
	goto	Ret2
	
Zero_	bcf	INTCON,T0IE	; If DSP1 & DSP2 = 0, disable interrupt
	bsf	PORTA,4		; Off relay
Ret2	movf    S_TEMP,w     	; Retrieve copy of STATUS register
	movwf	STATUS         	; Restore pre-isr STATUS register contents
	swapf   W_TEMP,f
	swapf   W_TEMP,w       	 ; Restore pre-isr W register contents
	retfie                 		 ; Return from interrupt

;**********************************************************************
;  	SubRoutine						      *	
;**********************************************************************
Disp	movf	DIGIT1,w		; Load DSP1  
	call	Table_		; Get LED data from table
	btfsc	FLAG_,7		; Blink dot-point on DSP1
	addlw	0x80	
	call	Send_Dat		; Send data to 74HC595
	bcf	DSP1		; On DSP1
	call	Delay2		; Delay for displaying DSP1
	bsf	DSP1		; Off DSP1

	movf	DIGIT2,w		; Load DSP2 data display 
	call	Table_		; Get LED data from table
	call	Send_Dat		; Send data to 74HC595
	bcf	DSP2		; On DSP2
	call	Delay2		; Delay for displaying DSP2
	bsf	DSP2		; Off DSP2
	return	

Send_Dat movwf	SEND		; Load data
	movlw	0x08		; Loop 8 times for sending 8-bit data
	movwf	COUNT
Chk_Bit	bcf	DAT_PIN		; Clear DATA pin of 74HC595 
	btfsc	SEND,7		; Check bit 7 "0" or "1"	
	bsf	DAT_PIN		; Send bit 7 to shift register
Clock	bsf	CLK		; Clock pulse
	bcf	CLK		
Rotate	rlf	SEND,f		; Rotate bit for sending next bit
	decfsz	COUNT,f		; Test loop 8 times ?
	goto	Chk_Bit		; If no, send bit again
Latch	bsf	LATCH		; Latch data for showing 
	bcf	LATCH
	return

Delay	movlw	0x50		; This delay use for waiting switch pressed and debounce
	movwf	DUMMY1		
	call	Disp
	decfsz	DUMMY1,f
	goto	$-2
	return	

Delay2	clrf	DUMMY3		; Multiplexing delay routine
	decfsz	DUMMY3,f
	goto	$-1	
	return
;**********************************************************************
;  	Read Write EEPROM Subroutine				      *	
;**********************************************************************
EE_Write bsf	EECON1,WREN	; Enable EEPROM writing
	movlw	0x55		; First command for EEPROM writing
	movwf	EECON2
	movlw	0xAA		; Second command 
	movwf	EECON2
	bsf	EECON1,WR	; Write EEPROM	
	bcf	EECON1,WREN	; Disable EEPROM writing
 	btfsc	EECON1,WR	; Test writing complete
	goto	$-1
	bcf	STATUS,RP0	
	return

EE_Read bsf	STATUS,RP0	; Load address for EEPROM reading
	movwf	EEADR
	bsf	EECON1,RD	; Read EEPROM
	movf	EEDATA,w	; Move data from EEPROM to W register
	bcf	STATUS,RP0
	return
	
	org	0x2100		; Default value of EEPROM
	de	0x05,0x08
	END                    		; Directive 'end of program'