;**********************************************************************
;    Filename:	    Exp15.asm                                        *
;    Date:          10/04/2002                                        *
;    Author: KRITSADA JAIYEN                                          *
;    Company: Innovative Experiment                                   *
;**********************************************************************
;    Files required: p16f628.inc                                      *
;**********************************************************************
;    Notes: LCD module demonstration			                      *
;**********************************************************************

	list      p=16f628     	; list directive to define processor
	#include <p16f628.inc> 	; processor specific variable definitions
	errorlevel 0,-302
	__CONFIG _CP_OFF & _WDT_OFF & _BODEN_ON & _PWRTE_ON & _XT_OSC & _MCLRE_ON & _LVP_OFF

	CBLOCK	0x20
BITS	
DUMMY1	
DUMMY2	
DAT	
COUNT	
	ENDC

#define	E	PORTB,0		
#define	RS	PORTB,3
	
	org	0x000		
	bsf	STATUS,RP0
	movlw	b'00000110'	; 6 output for controlling LCD module
	movwf	PORTB
	bcf	STATUS,RP0
	
Start	call	Init_LCD		; Initial LCD 
	bcf	E		; Set address for LCD first line
	bcf	RS		; Send command
	call	Delay125
	movlw	0x80		; 80 = 1'st line
	call	Send	
	bsf	RS
	call	Delay125
	movlw	0x10		; 16 character counter
	movwf	COUNT
	movlw	'A'		; Start with ASCII "A"
	movwf	DAT
_1st_Ln movf	DAT,w		; Send data to LCD module
	call	Send
	incf	DAT,f
	decfsz	COUNT,f		; Test counter = 16 times ?
	goto	_1st_Ln
	movlw	0x10		; If yes, send character to next line
	movwf	COUNT
	bcf	RS		; Send command
	call	Delay125		
	movlw	0xC0		; C0 is first address of 2nd line of LCD
	call	Send
	bsf	RS
	call	Delay125
_2nd_Ln movf	DAT,w		; Continue ASCII data
	call	Send
	incf	DAT,f
	decfsz	COUNT,f		; Test counter = 16 ?
	goto	_2nd_Ln	
	goto 	$		; If counter > 16 then halt

;**********************************************************************
;    	LCD MoDule Subroutine  			        *
;**********************************************************************
reset_LCD	macro	
	movlw	0x38		; Fisrt, send 8-bit mode command
	movwf	BITS
	call	Flip
	call	Pulse
	call	Delay125
	endm
Init_LCD bcf	E		; Set command mode
	bcf	RS
	call	Delay125
	reset_LCD
	reset_LCD
	reset_LCD
	reset_LCD
	movlw	0x28		; Send 4-bit mode command in MSB only
	movwf	BITS
	call	Flip
	call	Pulse
	call	Delay125
	movlw	0x28		; Set LCD mode to 4 bit mode, 2 line display, 5x7 dot
	call	Send
	movlw	0x0c		; Set LCD on, cursor off, cursor not blink
	call	Send
	movlw	0x01		; Clear display
	call	Send
	call	Delay5
	return
;**********************************************************************
;    	Send Data 8 Bit To LCD Module				      *
;**********************************************************************
Send	movwf	BITS		; Send 8-bit data to LCD module 
	call	Flip		; 
	call	Pulse		; Send Upper 4-bit first and lower 4-bit following
	swapf	BITS,f	
	call	Flip
	call	Pulse
	call	Delay125
	return
	
Flip	bcf	PORTB,4		; Copy data from BITS register to RB4-RB7
	btfsc	BITS,4		; 
	bsf	PORTB,4		; Unchange other bit of PORTB
	bcf	PORTB,5		; 
	btfsc	BITS,5
	bsf	PORTB,5
	bcf	PORTB,6
	btfsc	BITS,6
	bsf	PORTB,6
	bcf	PORTB,7
	btfsc	BITS,7
	bsf	PORTB,7
	return
	

Pulse	bsf	E		; Generate enable pluse for LCD
	nop
	bcf	E
	call	Delay125
	return

Delay125 movlw	.42		; Delay 125 microsecond
	movwf	DUMMY1
	decfsz	DUMMY1,f
	goto	$-1
	return

Delay5	movlw	.41		; Delay 5 millisecond
	movwf	DUMMY2
D1	call	Delay125
	decfsz	DUMMY2,f
	goto	D1
	return 	

		END                    	; Directive 'end of program'

