;**********************************************************************
;    Filename:	    Exp17.asm                                        *
;    Date:          16/04/2002                                        *
;    Author: KRITSADA JAIYEN                                          *
;    Company: Innovative Experiment                                   *
;**********************************************************************
;    Files required: p16f628.inc                                      *
;**********************************************************************
;    Notes: Recieve Data from RS-232 Port	              *
;**********************************************************************

	list      p=16f628            	; list directive to define processor
	#include <p16f628.inc>        	; processor specific variable definitions
	errorlevel 0,-302
	__CONFIG _CP_OFF & _WDT_OFF & _BODEN_ON & _PWRTE_ON & _XT_OSC & _MCLRE_ON & _LVP_OFF

W_TEMP	equ	0x20
S_TEMP	equ	0x21
DAT	equ	0x22
Flag_	equ	0x23
SEND	equ	0x24
COUNT	equ	0x25

#Define NEWDATA  Flag_,0
#define	DAT_PIN	PORTB,0		; 74595 DATA pin
#define LATCH	PORTB,4		; 74595 LATCH pin 
#define CLK	PORTB,5		; 74595 CLK pin
#Define	DSP1	PORTB,6		; DSP1 common
#Define	DSP2	PORTB,7		; DSP2 common

	org     0x000       	; Processor reset vector
  	goto    Init        	; Go to beginning of program

	org     0x004         	; Interrupt vector location
	goto	Isr

;**********************************************************************
;	INTERRUPT SERVICE ROUTINE 				      *	
;**********************************************************************
Isr	movwf   W_TEMP          ; Save current W register contents
	movf	STATUS,w        ; Move status register into W register
	bcf     STATUS,RP0      ; Ensure file register bank set to 0
	movwf	S_TEMP       	; Save contents of STATUS register
	movf	RCREG,W		; Read data to clear flag RCIF 
	movwf	DAT
	bsf	NEWDATA

	movf    S_TEMP,w     	; Retrieve copy of STATUS register
	movwf	STATUS         	; Restore pre-isr STATUS register contents
	swapf   W_TEMP,f
	swapf   W_TEMP,w       	; Restore pre-isr W register contents
	retfie      

;**********************************************************************
;	Initial Every Thing					      *
;**********************************************************************
Init 	bsf	STATUS,RP0	; Bank 1
	movlw	b'10001110'	; Use for sending serial data
	movwf	PORTB
	movlw	.25		; For 9600 bps 0.16% error baudrate
	movwf	SPBRG
	bcf	TXSTA,SYNC	; Set asynchonous Mode
	bsf	TXSTA,BRGH	; Set high speed baudrate
	bsf	TXSTA,TXEN	; Enable data transmittion
	bsf	PIE1,RCIE	; Enable receiver interrupt
	bcf	STATUS,RP0
	bcf	DSP1		; On LED for displaying receiving data
	bsf	INTCON,PEIE	; Enable periperal interrupt
	bsf	INTCON,GIE	; Enable global interrupt
	bsf	RCSTA,SPEN	; Enable serial Port
	bsf	RCSTA,CREN	; Enable receiving data

	movf	RCREG,W	; Flush data for first time
	movf	RCREG,W	; Use for clearing buffer
	movf	RCREG,W

;**********************************************************************
; 	START Program Here					      * 
;**********************************************************************
Wait_ST	bcf	NEWDATA	; Clear flag of testing new data
	btfss	NEWDATA	; If this flag set, we have the new data from computer
	goto	$-1
	movf	DAT,w		; Load data to W register 
	movwf	TXREG		; and send back to computer
	call	Send_Dat		; Show data on 8-bit  LED
	goto	Wait_ST		; Do loop 

Send_Dat movwf	SEND		; Load data
	movlw	0x08		; Loop 8 times for sending 8-bit data
	movwf	COUNT
Chk_Bit	bcf	DAT_PIN		; 
	btfsc	SEND,7		; Check bit 7 "0" or "1"	
	bsf	DAT_PIN		; Send bit 7 to shift register
Clock	bsf	CLK		; Clock pulse
	bcf	CLK		
Rotate	rlf	SEND,f		; Rotate bit for sending next bit
	decfsz	COUNT,f		; Test loop 8 times ?
	goto	Chk_Bit		; If no, send bit again
Latch	bsf	LATCH		; Latch data for displaying
	bcf	LATCH
	return
	
		end
