;**********************************************************************
;    Filename:	    Exp19.asm                                        *
;    Date:          18/04/2002                                        *
;    Author: KRITSADA JAIYEN                                          *
;    Company: Innovative Experiment                                   *
;**********************************************************************
;    Files required: p16f628.inc                                      *
;**********************************************************************
;    Notes: Incremnet Voltage Refference			      *
;**********************************************************************

	list      p=16f628            	; list directive to define processor
	#include <p16f628.inc>        	; processor specific variable definitions
	errorlevel 0,-302
	__CONFIG _CP_OFF & _WDT_OFF & _BODEN_ON & _PWRTE_ON & _XT_OSC & _MCLRE_ON & _LVP_OFF

COUNT	equ	0x20
SEND 	equ	0x21
DUMMY1	equ	0x22
DUMMY2	equ	0x23
TEMP	equ	0x70		; Access All Bank File Register

#define	DAT_PIN	PORTB,0		; 74595 DATA pin
#define LATCH	PORTB,4		; 74595 LATCH pin 
#define CLK	PORTB,5		; 74595 CLK pin

	org	0x000
	movlw	0x03		; RA3 as Digital I/O, other of port A as analog input
	movwf	CMCON
	bsf	STATUS,RP0
	movlw	b'10001110'	; Set port B to output for displaying on LED
	movwf	PORTB
	movlw	b'11100000'	; Set RA2 connect to reference voltage module in low range
	movwf	VRCON		; 
	bcf	STATUS,RP0
	clrf	TEMP		; Start with 0V

Loop	call	Delay		; Delay for debouncing switch 	
	btfsc	PORTA,3		; Test RA3 switch pressed ?
	goto	Loop		; If not, loop again
	call	Inc		; If yes, goto increment routine
	goto	Loop		; Check switch again

;**********************************************************************
;    	Subroutine						      *
;**********************************************************************
Inc	incf	TEMP,w		; Increment TEMP register value
	andlw	0x0f		; Use 4 LSB Bit
	movwf	TEMP		; Backup data		
	call	Send_Dat		; Show data to LED 
	bsf	STATUS,RP0	; Use TEMP register in all bank
	movlw	0xf0		; Maintain 4-bit MSB of VRCON register
	andwf	VRCON,w		; Clear 4-bit LSB of VRCON
	addwf	TEMP,w		; Add VRCON with TEMP
	movwf	VRCON		; Send back to VRCON
	bcf	STATUS,RP0
	return

Delay	clrf	DUMMY1		; Debouncing delay
Delay1	clrf	DUMMY2
	decfsz	DUMMY2,f
	goto	$-1
	decfsz	DUMMY1,f
	goto	Delay1
	return	

Send_Dat movwf	SEND		; Load data
	movlw	0x08		; Loop 8 times for sending 8-bit data
	movwf	COUNT
Chk_Bit	bcf	DAT_PIN		; 
	btfsc	SEND,7		; Check bit 7 "0" or "1"	
	bsf	DAT_PIN		; Send bit 7 to shift register
Clock	bsf	CLK		; Clock pulse
	bcf	CLK		
Rotate	rlf	SEND,f		; Rotate bit for sending next bit
	decfsz	COUNT,f		; Test loop 8 times ?
	goto	Chk_Bit		; If no, send bit again
Latch	bsf	LATCH		; Latch data for displaying
	bcf	LATCH
	return
	
	end
