;**********************************************************************
;    Filename:	    Exp21.asm                                         *
;    Date:          20/04/2002                                        *
;    Author: KRITSADA JAIYEN                                          *
;    Company: Innovative Experiment                                   *
;**********************************************************************
;    Files required: p16f628.inc                                      *
;**********************************************************************
;    Notes: Basic 1 Wire Temperature				      *
;**********************************************************************

	list      p=16f628    	; list directive to define processor
	#include <p16f628.inc> 	; processor specific variable definitions
	errorlevel 0,-302
	__CONFIG _CP_OFF & _WDT_OFF & _BODEN_ON & _PWRTE_ON & _XT_OSC & _MCLRE_ON & _LVP_OFF

DS_Dat	equ	0x20
DS_Sign	equ	0x21
DUMMY0	equ	0x22
DUMMY1	equ	0x23
DUMMY2	equ	0x24
SEND	equ	0x25
COUNT	equ	0x26

#define	DAT_PIN	PORTB,0		; 74595 DATA pin
#define LATCH	PORTB,4		; 74595 LATCH pin 
#define CLK	PORTB,5		; 74595 CLK pin
#Define	DQ	PORTA,0		; DS1820 I/O

	org	0x000
Start	clrf	PORTA		; Set port A for digital I/O
	movlw	0x07
	movwf	CMCON
	bsf	STATUS,RP0	
	movlw	b'10001110'	; Use port B for displaying data
	movwf	PORTB
	bcf	STATUS,RP0
;**********************************************************************
;    	Main Program						      *
;**********************************************************************
Main	call	RD_Temp	; Read temperature from DS1820
	movf	DS_Dat,w		; Load data to W register
	call	Send_Dat		; Send data
	goto	Main		; Loop again

;**********************************************************************
;    	Subroutine						      *
;**********************************************************************
Send_Dat movwf	SEND		; Load data
	movlw	0x08		; Loop 8 times for sending 8-bit data
	movwf	COUNT
Chk_Bit	bcf	DAT_PIN		; 
	btfsc	SEND,7		; Check bit 7 "0" or "1"	
	bsf	DAT_PIN		; Send bit 7 to shift register
Clock	bsf	CLK		; Clock pulse
	bcf	CLK		
Rotate	rlf	SEND,f		; Rotate bit for sending next bit
	decfsz	COUNT,f		; Test loop 8 times ?
	goto	Chk_Bit		; If no, send bit again
Latch	bsf	LATCH		; Latch data for displaying
	bcf	LATCH
	return

;**********************************************************************
;	Macro for DS1820					      *
;**********************************************************************
DQLOW	macro
	bcf	DQ		; DQ bit ready
	bsf	STATUS,RP0
	bcf	DQ		; Set DQ to output
	bcf	STATUS,RP0
	endm

DQHIZ	macro
	bsf	STATUS,RP0
	bsf	DQ		; Set DQ to input
	bcf	STATUS,RP0
	endm

PAUSE	macro	DELAY		; Generate delay time
	movlw	DELAY
	movwf	DUMMY0
	call	Delay5
	endm

;**********************************************************************
;    	DS1820 1-Wire Subroutine		                         *
;**********************************************************************
RD_Temp	call	DS_Rx		; Check chip status
	addlw	0x01		
	btfss	STATUS,Z	; Z set = ready
	return			; W is not zero = not ready

Get_Temp call	DS_Reset

	movlw	0xcc		; Skip ROM
	call	DS_Tx

	movlw	0xbe		; Read scratch pad RAM of DS1820
	call	DS_Tx
	call	DS_Rx		; Read 8-bit data 
	movwf	DS_Dat

	call	DS_Reset	; Restart\

	movlw	0xcc		; Skip ROM
	call	DS_Tx

	movlw	0x44		; Start conversion
	call	DS_Tx

Delay5	nop			; Delay for PAUSE  macro
	nop	
	decfsz	DUMMY0,f
	goto	Delay5
	return

DS_Reset DQLOW
	PAUSE	0x77		; 600 microcsecond delay
	DQHIZ
	PAUSE	0x0c		; Wait  for response pulse (67 microsecond)
	nop
	nop
	movf	PORTA,w		; Read response
	andlw	0x01		; Use RA0 only
	movwf	DUMMY1
	PAUSE	0x3b		; 300 microcsecond delay
	movf	DUMMY1,w	; Response in W
	return

DS_Tx	movwf	DUMMY2		; Transmission data 
	movlw	0x08		; Prepare 8-bit counter for sending data
	movwf	DUMMY1		; Define loop counter
Tx_Loop	DQLOW			; Macro of DQ pin to low, This is start bit
	PAUSE	0x01		; 10 microsecond delay
	rrf	DUMMY2,f	; Rotate data to Carry flag
	btfsc	STATUS,C	; Test Carry flag
	bsf	DQ		; If Carry flag = "1" , set DQ to high
	PAUSE	0x0d		; 70 microsecond delay
	DQHIZ
	nop
	decfsz	DUMMY1,f	; 8 times ?
	goto	Tx_Loop		; No, send again
	return

DS_Rx	movlw	0x08		; Recieve 8-bit data
	movwf	DUMMY1
Rx_Loop	DQLOW			; Macro of DQ pin to low, this is start bit
	PAUSE	0x01		; 10 microsecond delay
	DQHIZ			; Back to high for receiving
	nop
	nop
	movf	PORTA,w		; Read data
	andlw	0x01		; Get data bit 0 only
	addlw	0xff		; Move data bit 0 to Carry flag with addition method
	rrf	DUMMY2,f	; Move data bit 0 to DUMMY bit 7
	PAUSE	0x0b		; 60 microsecond delay
	decfsz	DUMMY1,f	; Loop 8 times
	goto	Rx_Loop		; Read again
	movf	DUMMY2,w	; Save data to W register
	return


	end
