;**********************************************************************
;                                                                     *
;    Filename:	    Exp23.asm                                         *
;    Date:          22/04/2002                                        *
;    Author: KRITSADA JAIYEN                                          *
;    Company: Innovative Experiment                                   *
;**********************************************************************
;    Files required: p16f628.inc                                      *
;**********************************************************************
;    Notes: Read Data From DS1820 And Show On LCD                     *
;**********************************************************************

	list      p=16f628            	; list directive to define processor
	#include <p16f628.inc>        	; processor specific variable definitions
	errorlevel 0,-302
	__CONFIG _CP_OFF & _WDT_OFF & _BODEN_ON & _PWRTE_ON & _XT_OSC & _MCLRE_ON & _LVP_OFF

	CBLOCK	0x20
DS_DAT	
DS_SIGN	
SEND	
COUNT	
NUM1	
NUM2	
NUM3	
DATA1	
HALF
DUMMY0
DUMMY1
DUMMY2
DUMMY5
DUMMY6
BITS
COMPARE	
	endc

#Define	DQ	PORTA,0		; DS1820 pin
#define	E	PORTB,0		; E pin of LCD module
#define	RS	PORTB,3		; RS pin of LCD module

	org	0x000	
	goto	Start		

Table_	addwf	PCL,f			; Set message table   
	dt	"<<Present Temp>>  "

Start	call	Init_Port	; Set-up port 
	call	Init_LCD	; Set-up LCD module 

Loop	movf	DS_DAT,w	; Save old data for comparison	
	movwf	COMPARE		 
	call	RD_Temp	; Read data from DS1820
	movf	COMPARE,w	; Compare with latest data
	subwf	DS_DAT,w	
	btfsc	STATUS,Z	; If old data = new data,  don't display again
	goto	Loop
	clrf	HALF		; Clear a half temp register
	bcf	STATUS,C
	rrf	DS_DAT,w	; Use 7-bit upper
	rlf	HALF,f		; Store a half degree (.5) or (.0)
	movwf	DATA1		; Data for converion to decimal
	call	Convert		; Convert data to decimal
;**********************************************************************
;    	Send Text From Table To LCD Module			
;**********************************************************************
	bcf	E		; Fix display position on LCD module
	bcf	RS		; Send command
	call	Delay125	
	movlw	0x80		; Defind afddress for sending command to LCD
	call	Send	
	bsf	RS
	call	Delay125		; End of sending command

	clrf	COUNT		; Get text from table
Send_Loop movf	COUNT,w		; Begin with counter = 0
	call	Table_		; Income data position 0
	call	Send		; Send data to LCD
	incf	COUNT,f		; Next data if Counter < 12
	movlw	.16
	subwf	COUNT,w		; Test counter < 12 ?
	btfss	STATUS,Z
	goto	Send_Loop	; If < 12, try again

;**********************************************************************
;    	Send Temperature To LCD					      *	
;**********************************************************************
	bcf	E		; Fix displkay position on LCD 
	bcf	RS		; Send command
	call	Delay125	
	movlw	0xC5		; Address for sending data to LCD
	call	Send	
	bsf	RS
	call	Delay125		; End of sending command

	movf	NUM3,w		; Load data from  DS1820
	btfsc	STATUS,Z	; Fisrt data = 0 ?
	movlw	0xe0		; Yes, convert to blank 0xE0+0x30 =0x20 (" ")
	addlw	0x30		; Convert number to ASCII		
	call	Send		; Send data to LCD
	movf	NUM2,w		; Load 2nd digit
	addlw	0x30		; Convert to ASCII
	call	Send		;
	movf	NUM1,w
	addlw	0x30
	call	Send
	movlw	'.'		; Send dot-point
	call	Send
	movlw	'0'
	btfsc	HALF,0		; Test half degree = "0"  or "5"
	movlw	'5'
	call	Send
	movlw	'C'
	call	Send
	goto	Loop	

;**********************************************************************
; 	SetPort I/O of PIC16F628				      *
;**********************************************************************
Init_Port clrf	PORTA		; Clear PORTA before initial
	movlw	0x07		; Set PORTA to digital I/O
	movwf	CMCON
	bsf	STATUS,RP0
	movlw	b'11111111'	; All PORTA are input
	movwf	PORTA
	movlw	b'00000110'	; Use PORTB for LCD
	movwf	PORTB
	bcf	STATUS,RP0
	return

;**********************************************************************
;	Read Data From DS1820 					      *	
;**********************************************************************
RD_Temp	call	DS_Rx		; Check DS1820 status
	addlw	0x01		; 255=ready, 0= ready 
	btfss	STATUS,Z	; Zero flag set = ready
	return			; W register is not zero = not ready
Get_Temp call	DS_Reset	; Reset chip first
	btfss	STATUS,Z	; Zero flag set = OK
	goto	Error_		; If not response, exit
	movlw	0xcc		; Skip ROM command
	call	DS_Tx		; Send command
	movlw	0xbe		; Read scratch pad command
	call	DS_Tx		; Send command
	call	DS_Rx		; Read 8-bit data
	movwf	DS_DAT		; Save data to register
	call	DS_Rx		; Sign (FF=-VE, 00=+VE)
	movwf	DS_SIGN		; Save 9th bit to register 
	call	DS_Reset	; Restart
	movlw	0xcc		; Skip ROM command
	call	DS_Tx		; Send command
	movlw	0x44		; Start convert command
	call	DS_Tx		; Send command

	movf	DS_SIGN,w	; Check dign 
	btfsc	STATUS,Z	; Check dign = 00 ?
	goto	Pass		; If yes, Temp as positive then return
	addlw	0x01		; 
	btfss	STATUS,Z	; Check sign = FF ?
	goto	Error_		; If not, end with error
Pass	retlw	0x00		; If Yes, (Temp as negstive then return	
Error_	retlw	0x01
	
;**********************************************************************
;	Macro for DS1820					      *
;**********************************************************************
DQLOW	macro
	bcf	DQ		; DQ bit ready
	bsf	STATUS,RP0
	bcf	DQ		; Set DQ to output
	bcf	STATUS,RP0
	endm

DQHIZ	macro
	bsf	STATUS,RP0
	bsf	DQ		; Set DQ to input
	bcf	STATUS,RP0
	endm

PAUSE	macro	DELAY		; Generate delay time
	movlw	DELAY
	movwf	DUMMY0
	call	DelayAL
	endm

;**********************************************************************
;    	DS1820 SubRoutine				   *
;**********************************************************************
DelayAL	nop			; Use for pausing macro
	nop
	decfsz	DUMMY0,f
	goto	DelayAL
	return

;**********************************************************************
;    	Reset DS1820						      *
;**********************************************************************
DS_Reset DQLOW
	PAUSE	0x77		; 600 microsecond delay
	DQHIZ
	PAUSE	0x0c		; Wait 67 microsecond for response bit
	nop
	nop
	movf	PORTA,w
	andlw	0x01		; Use RA0 only
	movwf	DUMMY1
	PAUSE	0x3b		; 300 microsecond delay
	movf	DUMMY1,w	; Response in W register
	return

;**********************************************************************
;    	Send Data 8 Bit To DS1820				      *
;**********************************************************************
DS_Tx	movwf	DUMMY2		; Transmission data 
	movlw	0x08		; Prepare 8-bit counter for sending data
	movwf	DUMMY1		; Define loop counter
Tx_Loop	DQLOW			; Macro of DQ pin to low, This is start bit
	PAUSE	0x01		; 10 microsecond delay
	rrf	DUMMY2,f	; Rotate data to Carry flag
	btfsc	STATUS,C	; Test Carry flag
	bsf	DQ		; If Carry flag = "1" , set DQ to high
	PAUSE	0x0d		; 70 microsecond delay
	DQHIZ
	nop
	decfsz	DUMMY1,f	; 8 times ?
	goto	Tx_Loop		; No, send again
	return	

;**********************************************************************
;    	Recieve Data 8 Bit From DS1820				      *
;**********************************************************************
DS_Rx	movlw	0x08		; Recieve 8-bit data
	movwf	DUMMY1
Rx_Loop	DQLOW			; Macro of DQ pin to low, this is start bit
	PAUSE	0x01		; 10 microsecond delay
	DQHIZ			; Back to high for receiving
	nop
	nop
	movf	PORTA,w		; Read data
	andlw	0x01		; Get data bit 0 only
	addlw	0xff		; Move data bit 0 to Carry flag with addition method
	rrf	DUMMY2,f	; Move data bit 0 to DUMMY bit 7
	PAUSE	0x0b		; 60 microsecond delay
	decfsz	DUMMY1,f	; Loop 8 times
	goto	Rx_Loop		; Read again
	movf	DUMMY2,w	; Save data to W register
	return

;**********************************************************************
;    	LCD MoDule Subroutine					      *
;**********************************************************************
reset_LCD	macro	
	movlw	0x38		; Fisrt, send 8-bit mode command
	movwf	BITS
	call	Flip
	call	Pulse
	call	Delay125
	endm
Init_LCD bcf	E		; Set command mode
	bcf	RS
	call	Delay125
	reset_LCD
	reset_LCD
	reset_LCD
	reset_LCD
	movlw	0x28		; Send 4-bit mode command in MSB only
	movwf	BITS
	call	Flip
	call	Pulse
	call	Delay125
	movlw	0x28		; Set LCD mode to 4 bit mode, 2 line display, 5x7 dot
	call	Send
	movlw	0x0c		; Set LCD on, cursor off, cursor not blink
	call	Send
	movlw	0x01		; Clear display
	call	Send
	call	Delay5
	return
;**********************************************************************
;    	Send Data 8 Bit To LCD Module				      *
;**********************************************************************
Send	movwf	BITS		; Send 8-bit data to LCD module 
	call	Flip		; 
	call	Pulse		; Send Upper 4-bit first and lower 4-bit following
	swapf	BITS,f	
	call	Flip
	call	Pulse
	call	Delay125
	return
	
Flip	bcf	PORTB,4		; Copy data from BITS register to RB4-RB7
	btfsc	BITS,4		; 
	bsf	PORTB,4		; Unchange other bit of PORTB
	bcf	PORTB,5		; 
	btfsc	BITS,5
	bsf	PORTB,5
	bcf	PORTB,6
	btfsc	BITS,6
	bsf	PORTB,6
	bcf	PORTB,7
	btfsc	BITS,7
	bsf	PORTB,7
	return
	

Pulse	bsf	E		; Generate enable pluse for LCD
	nop
	bcf	E
	call	Delay125
	return

Delay125 movlw	.42		; Delay 125 microsecond
	movwf	DUMMY1
	decfsz	DUMMY1,f
	goto	$-1
	return

Delay5	movlw	.41		; Delay 5 millisecond
	movwf	DUMMY2
D1	call	Delay125
	decfsz	DUMMY2,f
	goto	D1
	return 

;**********************************************************************
;    	Convert 8 bit Data To 3 Digit Decimal			      *
;**********************************************************************
Convert	clrf	NUM2		; Clear register of 10's unit
Check	movlw	0x0A		; Subtract with 10 until the result lower 10
	subwf	DATA1,w		; Fisrt subtraction < 10 ?
	btfss	STATUS,C	
	goto	Less1		; If < 10 then return
	incf	NUM2,f		; If > 10 then increase 10's unit
	movlw	0x0A		; Subtract with 10 
	subwf	DATA1,f		; Test Again. Is it lower 10 ? 
	goto	Check

Less1	movf	DATA1,w		; If < 10, send data to 1's unit 
	movwf	NUM1		; 
	clrf	NUM3		; Clear 100's unit
Check2	movlw	0x0A
	subwf	NUM2,w		; 10's unit > 10 ?
	btfss	STATUS,C
	return			; If < 10 then return
	incf	NUM3,f
	movlw	0x0A		; If > 10 then subtraction again
	subwf	NUM2,f		; and check until the result is lower 10
	goto	Check2

	end
