;**********************************************************************
;    Filename:	    Exp26.asm                                         *
;    Date:          25/04/2002                                        *
;    Author: KRITSADA JAIYEN                                          *
;    Company: Innovative Experiment                                   *
;**********************************************************************
;    Files required: p16f628.inc                                      *
;**********************************************************************
;    Notes: Show Pulse Width Modulation Frequency = 980 Hz            *
;           Use RA0 : Increment Duty Cycle			      *	
;		RA1 : Dedrement Duty Cycle                            *                             *
;**********************************************************************
	list      p=16f628      ; list directive to define processor
	#include <p16f628.inc>  ; processor specific variable definitions
	errorlevel 0,-302	; Disables Bank Warnning
	__CONFIG _CP_OFF & _WDT_OFF & _BODEN_ON & _PWRTE_ON & _XT_OSC & _MCLRE_ON & _LVP_OFF

	CBLOCK	0x20		; Define Block Of Constant Register Files Start Address 0x20
DUTY
DUMMY1
DUMMY2
	endc			; End Of Define

	org	0x000		; Reset Address

;**********************************************************************
;    	Set Port and Initial Data For PWM                             *
;**********************************************************************
Start	clrf	PORTA
	movlw	0x07		; Set PORTA to digital I/O
	movwf	CMCON
	bsf	STATUS,RP0
	movlw	b'11110111'	; RB3 as output
	movwf	PORTB		
	bcf	STATUS,RP0
	bcf	PORTB,3
	clrf	CCP1CON	; Capture/Compare/PWM Off
	bsf	STATUS,RP0
	movlw	0xff		; Period Time = 256
	movwf	PR2
	bcf	STATUS,RP0	; Frequency ~ 1 kHz
	bcf	CCP1CON,4
	bcf	CCP1CON,5
	movlw	0x7f		; Duty Cycle = 50%
	movwf	CCPR1L
	movlw	0x01
	movwf	T2CON		; Prescaler = 1:4 Timer2 Off
	clrf	TMR2
	movlw	b'00001100'
	movwf	CCP1CON	; Set PWM Mode
	bsf	T2CON,2		; Start Timer2

	movlw	0x7f		; Duty cycle = 50%
	movwf	DUTY

;**********************************************************************
;    	Check Key For Increment/Decrement Duty Cycle                  *
;**********************************************************************
Check	btfss	PORTA,0		; Check RA0 switch pressed ?
	call	Inc_
	btfss	PORTA,1		; Check RA1 switch pressed ?
	call	Dec_
	call	Delay200		; Delay for switch debouncing
	goto 	Check

Dec_	movf	DUTY,f		; Test DUTY = 0 ?
	btfsc	STATUS,Z
	return			; If DUTY = 0, return
	decf	DUTY,f		; If not, decrement
	movf	DUTY,w		; Move data from DUTY
	movwf	CCPR1L		; to CCPR1L register
	return

Inc_	movlw	0x01		; Test DUTY = FF ?
	addwf	DUTY,w		; FF+1 = 00 and Carry flag set
	btfsc	STATUS,C
	return			; If DUTY = FF, return 
	incf	DUTY,f		; If not, increment
	movf	DUTY,w		; Move data from DUTY
	movwf	CCPR1L		; to CCPR1L register
	return
		
Delay200 clrf	DUMMY1		; Delay is approximate 200 millisecond
	clrf	DUMMY2
	decfsz	DUMMY2,f
	goto	$-1
	decfsz	DUMMY1,f
	goto	$-3
	return

	end
